@@ -1,82 +1,126 @@
+/*
+ * Copyright (C) 2001, 2002, and 2003  Roy Keene
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 2.1
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ *      email: libopennet@rkeene.org
+ */
+
 
 /*
  *	Network related functions
  */
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include "conf.h"
+#include "compat.h"
 #include "opennet.h"
-
-#ifndef NO_NETWORK
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#include <stdarg.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <fcntl.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
 #include "parse.h"
 #include "crc.h"
 
-/*
-static int createlisten(int port);
-static void closeconnection(int sockfd);
-*/
-static int createconnection_tcp(char *host, int port);
+#ifndef O_BINARY
+#define O_BINARY 0x0
+#endif
+#ifndef S_IFSOCK
+#define S_IFSOCK 0140000
+#endif
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0x0
+#endif
 
-extern int errno;
-static char *opennet_urls[256];
 
-/*
- *	Create a listening port on tcp port PORT
- */
-/*
-static int createlisten(int port) {
-	struct sockaddr_in localname;
-	int sockFd;
-	sockFd=socket(AF_INET,SOCK_STREAM,IPPROTO_IP);
-	localname.sin_family=AF_INET;
-	localname.sin_port=htons(port);
-	localname.sin_addr.s_addr=INADDR_ANY;
-	if (bind(sockFd,(struct sockaddr *) & localname,sizeof(localname))==-1) { perror("bind");return(-1); }
-	if (listen(sockFd,1024)==-1) { perror("listen"); return(-1); }
-	return(sockFd);
+//extern int errno;
+struct opennet_url_info {
+	char *url;
+	int flags;
+	mode_t mode;
+};
+struct opennet_url_info opennet_urls[256];
+
+/* Do things required for network access. */
+int opennet_init(void) {
+#ifdef _USE_WIN32_  
+	WSADATA wsaData;
+
+	if (WSAStartup(MAKEWORD(2, 0), &wsaData)!=0) {
+		return(-1);                           
+	}
+	if (wsaData.wVersion!=MAKEWORD(2, 0)) {
+		/* Cleanup Winsock stuff */    
+		WSACleanup();              
+		return(-1);  
+	}
+#endif
+	return(0);
 }
-*/
-
-
-/*
- *	Close that socket, yeah.
- */
-/*
-static void closeconnection(int sockfd) {
-	shutdown(sockfd, 1);
-	close(sockfd);
-}
-*/
 
 static int createconnection_tcp(char *host, int port) {
 	int sockid;
 	struct hostent *hostinfo;
 	struct sockaddr_in sock;
-	if ((hostinfo=gethostbyname(host))==NULL) {
+
 #ifdef HAVE_INET_ATON
-		if (!inet_aton(host,&sock.sin_addr))
+	if (!inet_aton(host,&sock.sin_addr)) {
 #else
-		if ( (sock.sin_addr.s_addr=inet_addr(host) )==-1)
+#ifdef HAVE_INET_ADDR
+	if ( (sock.sin_addr.s_addr=inet_addr(host) )==-1) {
+#else
+	{
 #endif
-			return(-1);
-	} else {
+#endif
+		if ((hostinfo=gethostbyname(host))==NULL) return(-EPERM);
 		memcpy(&sock.sin_addr.s_addr,hostinfo->h_addr_list[0],hostinfo->h_length);
 	}
+
 	sock.sin_family=AF_INET;
 	sock.sin_port=htons(port);
-	if ((sockid=socket(AF_INET, SOCK_STREAM, 0))<0)
+	if ((sockid=socket(AF_INET, SOCK_STREAM, 0))<0) {
 		return(-1);
+	}
 	if (connect(sockid, (struct sockaddr *) &sock, sizeof(sock))<0) {
 		close(sockid);
 		return(-1);
@@ -84,28 +128,19 @@ static int createconnection_tcp(char *host, int port) {
 	return(sockid);
 }
 
-int open_net(const char *pathname, int flags, ...) {
+int open_net(const char *pathname, int flags, mode_t mode) {
 	char scheme[5], host[512], file[1024];
 	char username[128], password[128];
 	char *read_buf, *read_buf_s=NULL;
 	char *smallbuf, *tmpbuf;
 	int port, fd, x, ftpfd=-1, retrmode=0;
-	va_list args;
-	mode_t mode=0644;
-
-	if (flags==O_WRONLY || flags==O_RDWR) {
-		va_start(args, flags);
-		/* This *SHOULD* be `int' here, not mode_t, for some reason. */
-		mode = va_arg(args, int);
-		va_end(args);
-	}
 
 	if (!parse_url(pathname,scheme,username,password,host,&port,file)) {
 		if ((fd=createconnection_tcp(host,port))<0) return(-1);
 
-		switch (ELFCRC(0, scheme, strlen(scheme))) {
+		switch (elfcrc(0, scheme, strlen(scheme))) {
 			case 457648: /* http */
-				if ((flags&3)==O_WRONLY || (flags&3)==O_RDWR) {
+				if ((flags&O_WRONLY)==O_WRONLY || (flags&O_RDWR)==O_RDWR) {
 					close(fd);
 					return(-1);
 				}
@@ -134,8 +169,8 @@ int open_net(const char *pathname, int flags, ...) {
 				if (strncmp("200 ",read_buf,4) && strncmp("302 ",read_buf,4)) {
 					free(read_buf_s);
 					close(fd);
-					errno=ENOENT;
-					return(-1);
+//					errno=ENOENT;
+					return(-ENOENT);
 				}
 				retrmode=read_buf[0];
 				read_buf=read_buf_s;
@@ -158,12 +193,14 @@ int open_net(const char *pathname, int flags, ...) {
 
 				free(read_buf_s);
 
-				if (opennet_urls[fd]!=NULL) free(opennet_urls[fd]);
-				opennet_urls[fd]=strdup(pathname);
+				if (opennet_urls[fd].url!=NULL) free(opennet_urls[fd].url);
+				opennet_urls[fd].url=strdup(pathname);
+				opennet_urls[fd].flags=flags;
+				opennet_urls[fd].mode=mode;
 				return(fd);
 				break;
 			case 28080: /* ftp */
-				if ((flags&3)==O_RDWR) {
+				if ((flags&O_RDWR)==O_RDWR) {
 					close(fd);
 					return(-1);
 				}
@@ -175,35 +212,35 @@ int open_net(const char *pathname, int flags, ...) {
 					if ((x=read(fd, read_buf, 1024))<1024) {
 						if (x<0) {
 							free(read_buf_s);
-							errno=EIO;
-							return(-1);
+//							errno=EIO;
+							return(-EIO);
 						}
 						/* It's waiting for input... */
 					}
 					read_buf[x]=0;
 
 					while ((smallbuf=strsep(&read_buf,"\n"))!=NULL) {
-						switch (ELFCRC(0, smallbuf, 4)) {
+						switch (elfcrc(0, smallbuf, 4)) {
 							case 231456: /* 550 */
 								if (ftpfd!=-1) close(ftpfd);
 								close(fd);
 								free(read_buf_s);
-								errno=ENOENT;
-								return(-1);
+//								errno=ENOENT;
+								return(-ENOENT);
 								break;
 							case 230944: /* 530 */
 								if (ftpfd!=-1) close(ftpfd);
 								close(fd);
 								free(read_buf_s);
-								errno=EIO;
-								return(-1);
+//								errno=EIO;
+								return(-EIO);
 								break;
 							case 231504: /* 553 */
 								if (ftpfd!=-1) close(ftpfd);
 								close(fd);
 								free(read_buf_s);
-								errno=EPERM;
-								return(-1);
+//								errno=EPERM;
+								return(-EPERM);
 								break;
 							case 218400: /* 220 */
 								write(fd, "USER ", 5);
@@ -235,19 +272,21 @@ int open_net(const char *pathname, int flags, ...) {
 								write(fd, "TYPE I\n", 7);
 								break;
 							case 217888:
-								if ((flags&3)==O_RDONLY) write(fd,"RETR ",5);
-								if ((flags&3)==O_WRONLY) write(fd,"STOR ",5);
+								if ((flags&O_RDONLY)==O_RDONLY) write(fd,"RETR ",5);
+								if ((flags&O_WRONLY)==O_WRONLY) write(fd,"STOR ",5);
 								write(fd, file, strlen(file));
 								write(fd, "\n", 1);
 								if ((ftpfd=createconnection_tcp(host,port))<0) {
 									close(fd);
-									errno=ENOENT;
-									return(-1);
+//									errno=ENOENT;
+									return(-ENOENT);
 								}
 								break;
 							case 215072: /* 150 */ 
-								if (opennet_urls[ftpfd]!=NULL) free(opennet_urls[ftpfd]);
-								opennet_urls[ftpfd]=strdup(pathname);
+								if (opennet_urls[ftpfd].url!=NULL) free(opennet_urls[ftpfd].url);
+								opennet_urls[ftpfd].url=strdup(pathname);
+								opennet_urls[ftpfd].flags=flags;
+								opennet_urls[ftpfd].mode=mode;
 								return(ftpfd);
 								break;
 							case 218656: /* 230 */
@@ -267,9 +306,11 @@ int open_net(const char *pathname, int flags, ...) {
 		free(read_buf_s);
 		close(fd);
 	} else {
-		fd=open(pathname,flags,mode);
-		if (opennet_urls[fd]!=NULL) free(opennet_urls[fd]);
-		opennet_urls[fd]=strdup(pathname);
+		fd=open(pathname,flags|O_BINARY|O_LARGEFILE,mode);
+		if (opennet_urls[fd].url!=NULL) free(opennet_urls[fd].url);
+		opennet_urls[fd].url=strdup(pathname);
+		opennet_urls[fd].flags=flags;
+		opennet_urls[fd].mode=mode;
 		return(fd);
 	}
 
@@ -278,52 +319,37 @@ int open_net(const char *pathname, int flags, ...) {
 
 off_t lseek_net(int filedes, off_t offset, int whence) {
 	struct stat file_status;
+	char tmpbuf[1024];
+	off_t retval, rset=0, i;
+	size_t n;
+	ssize_t read_val;
 	int newfd;
 
 	fstat(filedes, &file_status);
-	if (S_ISSOCK(file_status.st_mode)) {
-		if (whence==SEEK_END) return(-1);
-		if (offset!=0 || whence!=SEEK_SET) return(-1); /* For now ... */
-		if ((newfd=open_net(opennet_urls[filedes],O_RDONLY))<0) return(-1);
-		close(filedes);
-		dup2(newfd,filedes);
-		return(0);
-	}
-	return(lseek(filedes,offset,whence));
-}
-
-ssize_t read_net(int fd, void *buf, size_t count) {
-	ssize_t i=0,x;
-
-	while (i!=count) {
-		x=read(fd,(void *) ((buf)+i),count-i);
-		if (x==0) break;
-		if (x==-1) return(-1);
-		i+=x;
+	if ((file_status.st_mode&S_IFSOCK)==S_IFSOCK) {
+		if (whence!=SEEK_CUR || offset<=0) {
+			if (whence!=SEEK_SET) return(-1); /* For now ... */
+			if (opennet_urls[filedes].url==NULL) return(-1);
+			if ((newfd=open_net(opennet_urls[filedes].url,opennet_urls[filedes].flags,opennet_urls[filedes].mode))<0) return(-1);
+			close(filedes);
+			dup2(newfd,filedes);
+			if (offset==0) return(0);
+		}
+	} else {
+		retval=lseek(filedes, offset, whence);
+		if (!(retval<0 && whence==SEEK_CUR && offset>0)) {
+			return(retval);
+		}
 	}
 
-	return(i);
+	retval=0;
+	for (i=0; i<offset; i++) {
+		n=offset-rset;
+		if (n>sizeof(tmpbuf)) n=sizeof(tmpbuf);
+		read_val=read(filedes, tmpbuf, n);
+		if (read_val<=0) { retval=-1; break; }
+		rset+=read_val;
+	}
+	retval=offset+1;
+	return(retval);
 }
-
-
-#else
-static int createlisten(int port) { return(-1); }
-static void closeconnection(int sockfd) { return; }
-static int createconnection_tcp(char *host, int port) { return(-1); }
-int open_net(const char *pathname, int flags,...) {
-	va_list args;
-	mode_t mode;
-
-	va_start(args, flags);
-	mode = va_arg(args, mode_t);
-	va_end(args);
-
-	return(open(pathname,flags,mode));
-}
-off_t lseek_net(int filedes, off_t offset, int whence) {
-	return(lseek(filedes,offset,whence));
-}
-ssize_t read_net(int fd, void *buf, size_t count) {
-	return(read(fd, buf, count));
-}
-#endif
